@@ -116,7 +116,7 @@ example_csh_normalize_contact (TpHandleRepoIface *repo,
 
   if (id[0] == '\0')
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "ID must not be empty");
       return NULL;
     }
@@ -125,28 +125,28 @@ example_csh_normalize_contact (TpHandleRepoIface *repo,
 
   if (at == NULL || at == id || at[1] == '\0')
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "ID must look like aaa@bbb");
       return NULL;
     }
 
   if (strchr (at + 1, '@') != NULL)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "ID cannot contain more than one '@'");
       return NULL;
     }
 
   if (at[1] == '#' && at[2] == '\0')
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "chatroom name cannot be empty");
       return NULL;
     }
 
   if (strchr (at + 2, '#') != NULL)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "realm/chatroom cannot contain '#' except at the beginning");
       return NULL;
     }
@@ -164,20 +164,20 @@ example_csh_normalize_room (TpHandleRepoIface *repo,
 
   if (id[0] != '#')
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "Chatroom names in this protocol start with #");
     }
 
   if (id[1] == '\0')
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "Chatroom name cannot be empty");
       return NULL;
     }
 
   if (strchr (id, '@') != NULL)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "Chatroom names in this protocol cannot contain '@'");
       return NULL;
     }
