@@ -24,7 +24,8 @@ typedef enum
   TP_DEBUG_MANAGER       = 1 << 7,
   TP_DEBUG_CHANNEL       = 1 << 8,
   TP_DEBUG_PROXY         = 1 << 9,
-  TP_DEBUG_HANDLES       = 1 << 10
+  TP_DEBUG_HANDLES       = 1 << 10,
+  TP_DEBUG_CONTACTS      = 1 << 11
 } TpDebugFlags;
 
 gboolean _tp_debug_flag_is_set (TpDebugFlags flag);
